<?php
/**
	@file campoFecha.php
	Fichero de la clase del campoFecha.
	@copy Copyright Ilke Benson 2008
	@author Miguel Jaque Barbero (mjaque@ilkebenson.com)
	@version 1.0
	@date 2008-02-19
**/

class CampoFecha extends Campo{
/**-	Representa un campo de fecha. 
	El campo de fecha permite al usuario pulsar sobre él y seleccionar la fecha de un calendario.
	Está formado por un span que contiene un campo de texto y un script.
	Para el funcionamiento del campo Fecha, es necesario que el documento XHTML cargue los scripts correspondientes.
**/
	const FORMATODEFECTO = 'd/m/Y';	///< Formato por defecto.
	protected $atributos = array('id','valor','clase','tabindex','formato');		///< Array de atributos del campo.

	public function __construct($nodo){
	/**-	Construye el campoFecha
		Los atributos operativos son:
			- id: Identificador que se asignará al campo.
			- clase: Clase de CSS que se asignará al campo.
			- formato: Formato de la fecha. Por defecto 'd/m/Y'.
			- valor: Valor de la fecha. Si se asigna el valor 'hoy' se asigna la fecha de hoy.
			- tabindex: Valor del índice de tabulación.
		@param $nodo Nodo fecha de PAX (<pax:fecha .../>).
	**/
		parent::__construct($nodo);
		
		if (!isset($this->id)) throw new ExcepcionCampoAtributoObligatorioNoDefinido('id',get_class($this));
		if (!isset($this->formato)) $this->formato = self::FORMATODEFECTO;	// Valor por defecto
		if ($this->valor == 'hoy')
			$this->valor = date($this->formato);
	}
	public function verNodo(){
	/**-	Devuelve el nodo que constituye el campo.
		@return DOMElement con el nodo que representa el campo
	**/
		$xml = new DOMDocument();
		// Creamos el nodo del campo de texto
		$input = $xml->createElement("input");
		$input->setAttribute("type", "text");
		$input->setAttribute("size", "10");
		$input->setAttribute("maxlength", "10");
		$input->setAttribute("id", $this->id);
		$input->setAttribute("name", $this->id);
		$input->setAttribute("value", $this->valor);
		if (isset($this->clase))
			$input->setAttribute("class", $this->clase);
		if (isset($this->tabindex))
			$input->setAttribute("tabindex", $this->tabindex);

		// Creamos el nodo de script
		$script = $xml->createElement("script");
		$script->setAttribute("type", "text/JavaScript");
		$codigo = '	Calendar.setup({';
		$codigo .= '	inputField : "'.$this->id.'",';
                //codigo .= '	ifFormat   : "%Y-%m-%d",';	//OJO, debería formarse utilizando $this->formatoFecha
                $codigo .= '	ifFormat   : "%d/%m/%Y",';	//OJO, debería formarse utilizando $this->formatoFecha
                $codigo .= ' 	button     : "'.$id.'"})';	
		$script->appendChild($xml->createTextNode($codigo));

		// Creamos un span para unirlos
		$span = $xml->createElement("span");
		$span->appendChild($input);
		$span->appendChild($script);

		return $span;
	}
}

///	Excepciones.

return true;	//Indicador de carga para __autoload
?>
