/**
	@file iu.js
	Fichero con varias clases, constantes y funciones.
	Contiene:
		Función $(), sustituo de document.getElementById().
		Constantes de PAX
		Clases Lista, Ajax y Parámetro
	@copy Copyright Ilke Benson 2008
	@author Miguel Jaque Barbero (mjaque@ilkebenson.com)
	@version 1.0
	@date 2008-02-06

**/
/**	Función auxiliar para facilitar la escritura de 'document.getElementById'.
	Devuelve una referencia al elemento identificado por id.
	@param id Nombre del identificador del elemento buscado.
	@return La referencia al elemento (lo mismo que document.getElementById)
**/
function $(id){
	return document.getElementById(id);
}
// Constantes de PAX{
const PAX_ASYNC = true; 
const PAX_SYNC = false;
//const PAX_OK = "Ok";
const PAX_ERROR = "Error";
//const PAX_MSG_TIMEOUT = "El servidor no responde";
//const PAX_MSG ="Se ha producido un error. \nReintente la operación y si el problema persiste, contacte con el administrador informándole del siguiente texto:\n";
const PAX_TEXTO = "texto";
const PAX_XML = "xml";
const PAX_JSON = "json";
//const PAX_EXCP_ERROR = "PAX_EXCP_ERROR";
//const PAX_COMM_OK = 1;
//const PAX_COMM_AVISO = 2;
//const PAX_COMM_ERROR = 3;
//const PAX_COMM_DESASTRE = 4;
//const PAX_ESTADO_ESPERAR = 0;
//const PAX_ESTADO_CREAR = 1;
//const PAX_ESTADO_EDITAR = 2;
//const PAX_ESTADO_ELIMINAR = 3;
//const PAX_CABECERA_XML = "<?xml version=\"1.0\" encoding=\"iso-8859-15\"?>";
//}
function Iu(){
	const DIV_AVISO = "avisos";	// Identificador del Div de Avisos
	const DIV_EXCEPCION = "excepcion";	// Identificador del Div de Excepciones
	this.comunicar = function(tipo, mensaje){
		$(DIV_AVISO).setAttribute("class", tipo);
		$(DIV_AVISO).innerHTML = mensaje;
	}
	this.notificarExcepcion = function(excepcion){
		while ($(DIV_EXCEPCION).hasChildNodes())
			$(DIV_EXCEPCION).removeChild($(DIV_EXCEPCION).firstChild);
		h1 = document.createElement("h1");
		titulo = document.createTextNode("Se ha producido un Error Local");
		h1.appendChild(titulo);
		p1 = document.createElement("p");
		texto = document.createTextNode(excepcion.toString().split(":")[1]);
		p1.appendChild(texto);
		p4 = document.createElement("p");
		p4.setAttribute("id", "botonExcepcion");
		boton = document.createElement("input");
		boton.setAttribute("type", "button");
		boton.setAttribute("value", "Cerrar");
		boton.setAttribute("onclick", "iu.cerrarExcepcion()");
		p4.appendChild(boton);
		$(DIV_EXCEPCION).appendChild(h1);
		$(DIV_EXCEPCION).appendChild(p1);
		$(DIV_EXCEPCION).appendChild(p4);
		$(DIV_EXCEPCION).style.display = 'block';
		if (!MODO_DESARROLLO) return true;	//Evita que se propague
	}
	this.notificarExcepcionServidor = function(excepcion){
		while ($(DIV_EXCEPCION).hasChildNodes())
			$(DIV_EXCEPCION).removeChild($(DIV_EXCEPCION).firstChild);
		h1 = document.createElement("h1");
		titulo = document.createTextNode(excepcion.titulo);
		h1.appendChild(titulo);
		p1 = document.createElement("p");
		texto = document.createTextNode(excepcion.texto);
		p1.appendChild(texto);
		p3 = document.createElement("p");
		solucionUsuario = document.createTextNode(excepcion.solucionUsuario);
		p3.appendChild(solucionUsuario);
		p4 = document.createElement("p");
		p4.setAttribute("id", "botonExcepcion");
		boton = document.createElement("input");
		boton.setAttribute("type", "button");
		boton.setAttribute("value", "Cerrar");
		boton.setAttribute("onclick", "iu.cerrarExcepcion()");
		p4.appendChild(boton);
		$(DIV_EXCEPCION).appendChild(h1);
		$(DIV_EXCEPCION).appendChild(p1);
		if (MODO_DESARROLLO){
			p2 = document.createElement("p");
			solucionProgramador = document.createTextNode(excepcion.solucionProgramador);
			p2.appendChild(solucionProgramador);
			$(DIV_EXCEPCION).appendChild(p2);
		}
		$(DIV_EXCEPCION).appendChild(p3);
		$(DIV_EXCEPCION).appendChild(p4);
		$(DIV_EXCEPCION).style.display = 'block';
	}
	this.cerrarExcepcion = function(){
		$(DIV_EXCEPCION).style.display = 'none';
	}
	this.cambiarCursor = function(tipo){
	/**-	Cambia el tipo del cursor
		@param tipo Texto con el tipo del cursor (wait, default, auto, ...)
	**/
		body = document.getElementsByTagName('body').item(0);
		body.style.cursor = tipo;
	}
	this.imprimir = function(){
	/**-	Imprime la página actual.
	**/
		window.print();
	}
}
function Lista(){
///	Clase de Interfaz de Usuario que representa un campo de lista desplegable (select).
	this.elementoXML = 'item';	//Nombre del elemento XML de cada item de la lista.
	this.valor;			//Valor del campo.
	
	this.cargar = function(datos){
	/**-	Carga la lista apartir de un array de datos.
		@param datos Array de objetos. Cada objeto debe tener los atributos "valor" y "texto". 
	**/
		this.limpiar();
		if (this.nulo)
			this.pon("Sin definir", "-1");
		if (datos.length)
			for (var i=0; i<datos.length; i++)
				this.pon(datos[i].texto, datos[i].valor);
		this.seleccionarPorValor(this.valorInicial);
	}
	this.cargarListaAuxiliar = function(listaAuxiliar){
	/**-	Carga la lista apartir de una respuesta AJAX.
		@param listaAuxiliar El objeto respuesta de AJAX, un objeto tipo ListaAuxiliar con un elemento items que contiene un array de item, cada uno de ellos con los atributos id y nombre.
	**/
		this.limpiar();
		lista = listaAuxiliar.items[0].item;
		if (this.nulo) this.pon("Sin definir", "-1");
		if (lista.length)
			for (var i=0; i<lista.length; i++)
				this.pon(lista[i].nombre, lista[i].id);
		this.seleccionarPorValor(this.valorInicial);
	}
	this.pon = function(texto,valor){
		var opcion = document.createElement("option");
		opcion.setAttribute("value", valor);
		opcion.setAttribute("tooltiptext", texto);
		nodoTexto = document.createTextNode(texto);
		opcion.appendChild(nodoTexto);
		//this.campo.appendChild(opcion);
		this.appendChild(opcion);
		//this.campo.selectedIndex = 0;
	}
	this.limpiar = function(){
	/**-	Elimina todas las opciones de la lista.
	**/
		//while(this.campo.hasChildNodes())
		while(this.hasChildNodes())
			//this.campo.removeChild(this.campo.firstChild);
			this.removeChild(this.firstChild);
	}
	this.seleccionarPorValor = function(valor){
	/**-	Selecciona la opción de la lista con el valor indicado.
		La selección es por valor, no por el texto de la opción. Si varias opciones tienen el mismo valor, se selecciona la primera de ellas.
		@param valor Valor de la opción que se quiere seleccionar.
	**/
		for(var i=0; i<this.options.length; i++)
			if (this.options[i].value == valor){
				this.selectedIndex = this.options[i].index;
				this.valor = this.options[i].value;
				return;
			}
		this.selectedIndex = -1;
	}
	this.seleccionarPorTexto = function(texto){
	/**-	Selecciona la opción de la lista con el texto indicado.
		La selección es por texto, no por el valor de la opción. Si varias opciones tienen el mismo texto, se selecciona la primera de ellas.
		@param texto Texto de la opción que se quiere seleccionar.
	**/
		for(var i=0; i<this.options.length; i++)
			if (this.options[i].text == texto){
				this.selectedIndex = this.options[i].index;
				this.valor = this.options[i].value;
				break;
			}
	}
	this.alCambiar = function(){
		this.valor = this.options[this.selectedIndex].value;
	}
	// Eventos de la clase
	if (this.addEventListener){
		this.addEventListener('change',this.alCambiar,false);
	}
	else{ //Para IE
		this.attachEvent('onchange',this.buscar);
		this.attachEvent('onkeypress',this._borrar);
	}

	//Establecemos el valor inicial, si lo tiene.
	if (this.getAttribute('valor')){
		this.seleccionarPorValor(this.getAttribute('valor'));
		this.onchange();	//Lanzamos el evento
	}
}
function ListaDesplegableAmpliable(){
/**-	Clase de interfaz que representa una lista ampliable.
**/
	this.id = this.attributes.getNamedItem("id").nodeValue;
	this.input = this.childNodes.item(0);
	this.input.listaAmpliable = this;
	this.select = this.childNodes.item(1);
	this.select.listaAmpliable = this;
	this.boton = this.childNodes.item(2);
	this.boton.listaAmpliable = this;
	Lista.call(this.select);		//Sobrecargamos el select con la clase Lista.
	this.valor;
	this.imagenEditar = 'iu/img/edit.png';
	this.imagenCancelar = 'iu/img/editdelete.png';

	this.cambiar = function(){
	/**-	Cambia el estado de la lista mostrando el input si se mostraba la lista y viceversa.
	 **/
	 	if (this.input.style.display == 'none'){
			this.select.style.display = 'none';
			this.input.style.display = 'inline';
			this.boton.src = this.imagenCancelar;
		}
		else{
			this.input.style.display = 'none';
			this.select.style.display = 'inline';
			this.boton.src = this.imagenEditar;
		}
	}
	this.limpiar = function(){
	/**-	Elimina todas las opciones de la lista, borra el campo de texto y pone la visualización inicial.
	**/
		this.valor = "";
		this.input.value = "";
		this.select.limpiar();
		this.input.style.display = 'none';
		this.select.style.display = 'inline';
		this.boton.src = this.imagenEditar;
	}
	this.alPulsar = function(){
		this.listaAmpliable.cambiar();
	}
	this.alSeleccionar = function(){
	/**-	Evento que atiende la modificación de la lista.
		Establece el valor del objeto.
	**/
		this.listaAmpliable.valor = this.options[this.selectedIndex].value;
	}
	this.alCambiarTexto = function(){
	/**-	Evento que atiende la modificación del valor del campo de texto.
		Establece el valor del objeto.
	**/
		this.listaAmpliable.valor = this.value;
	}
	this.alCambiarValor = function(atributo, valorActual, valorNuevo){
	/**-	Evento que atiende la modificación del atributo valor.
	**/
		this.input.value = valorNuevo;
		this.select.seleccionarPorValor(valorNuevo);
		this.valor = valorNuevo;
		if(this.onchange)	//Lanzamos el evento, si lo hay.
			this.onchange();
		return valorNuevo;	//Para hacer efectivo el cambio de valor.
	}
	
	// Eventos de la clase
	if (this.boton.addEventListener){
		this.boton.addEventListener('click',this.alPulsar,false);
		this.select.addEventListener('change',this.alSeleccionar,false);
		this.input.addEventListener('change',this.alCambiarTexto,false);
	}
	else{ //Para IE
		this.input.attachEvent('onclick',this.alPulsar);
	}
	
	// Seguimiento de cambios al atributo valor
	this.watch('valor', this.alCambiarValor);

	// Establecemos el estado inicial
	this.input.style.display = 'none';
	this.select.style.display = 'inline';

	//Establecemos el valor inicial
	if (this.getAttribute('valor'))
		this.valor = this.getAttribute("valor");
}
function Ajax(url){
/**-	Clase que implementa una comunicación AJAX.
**/

	this.url = url;	//La url del servidor.
	
	this.pedir = function (accion, parametros, formatoRespuesta, async, manejador, entorno){
	/**-	Realiza una petición AJAX al servidor.
	 	El tratamiento de la respuesta y de los errores lo realiza la función global atenderRespuestaAJAX.
		@param accion Nombre de la acción a ejecutar, en el formato 'objeto.método'.
		@param parametros Array de objetos Parametro con los parámetros requeridos para la acción.
		@param formatoRespuesta Texto indicando el formato en el que se solicita la respuesta del servidor (XML, JSON, Texto).
		@param async Indica si la acción es síncrona o asíncrona. Si es síncrona, ejecutar devuelve el resultado. Si es asíncrona, pasará el resultado al manejador.
		@param manejador Nombre de la función (objeto.método) a la que se pasará el resultado si la llamada es asícrona.
		@param entorno Referencia al objeto que se pasará como entorno de trabajo al manejador.
		@return Si la llamada es síncrona, ejecutar devuelve la respuesta del servidor. En caso contrario, no devuelve nada.
	**/
		
		//Código de http://developer.mozilla.org/en/docs/AJAX:Getting_Started
		var peticion;

		if (window.XMLHttpRequest) { // Mozilla, Safari, ...
			peticion = new XMLHttpRequest();
			if (peticion.overrideMimeType) {
				peticion.overrideMimeType('text/xml');
			}
		} 
		else if (window.ActiveXObject) { // IE
			try {
				peticion = new ActiveXObject("Msxml2.XMLHTTP");
			} 
			catch (e) {
				try {
					peticion = new ActiveXObject("Microsoft.XMLHTTP");
				} 
				catch (e) {}
			}
		}
		if (!peticion) {
			alert('Su navegador no tiene un soporte conocido para AJAX. No es posible comunicar con el servidor.');
			return false;
		}
		peticion.manejador = manejador;
		peticion.entorno = entorno;
		peticion.async = async;
		peticion.formatoRespuesta = formatoRespuesta;
		peticion.onerror = function(){throw("Error de AJAX al ejecutar " + accion);};
		if (async) peticion.addEventListener("load", function(){iu.ajax.atenderRespuesta.call(iu.ajax, peticion);}, false);
		peticion.open('POST', this.url, async);		//Peticiones posteriores, no se concatenan a la base de datos

		peticion.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");
		var mensaje = "PaxAccion=" + accion;
		
		mensaje += this.verParametros(parametros);
	
		if (formatoRespuesta == '') formatoRespuesta = PAX_XML;	// Por defecto, es AJAX 
		mensaje += "&PaxFormatoRespuesta=" + formatoRespuesta;
		
		respuesta = peticion.send(mensaje);		//Enviamos la petición.

		if(!async) return this.atenderRespuesta(peticion);

	};
	this.verParametros = function (parametros){
	/**-	Devuelve el texto de URL para transmitir los parámetros por GET.
		@param parametros Array de objetos Parametro.
		@return Texto para anexar a la URL.
	**/
		mensaje = "";
		if (parametros)
			for (var i=0; i<parametros.length; i++)
				mensaje += "&" + parametros[i].nombre+ "=" + parametros[i].valor;
		return mensaje;
	}
	this.irA = function (accion,parametros){
	/**-	Carga una nueva página llamando a la acción indicada con los parámetros indicados.
		@param accion Nombre de la acción.
		@param parametros Texto con los parámetros a enviar en formato POST. 
		Cada parámetro debe indicarse precedido de & como separador, su nombre y su valor (&nombre=miNombre&clave=miClave)
	**/
		window.location.href = this.url+"?PaxAccion="+accion+this.verParametros(parametros);
	};
	this.atenderRespuesta = function(peticion){
	/**-	Función global para atender las respuestas AJAX. Esta función es llamada en el evento onreadystatechange de la petición AJAX.
		Analiza los posibles errores y, si todo es correcto, llama al manejador con la respuesta.
		@param peticion Petición XMLHttpRequest realizada. Incluye atributos añadidos al realizar la petición.
	**/
		//alert(peticion.responseText);
		if (peticion.readyState == 4) { //Petición Completada
			if (peticion.status == 200) {
				//Tratamiento de la respuesta.
				var respuesta;
				switch (peticion.formatoRespuesta){
					case (PAX_TEXTO):
						respuesta = peticion.responseText;
						break;
					case (PAX_JSON):
						respuesta = eval(peticion.responseText);
						break;
					case (PAX_XML):
					default:
						if (peticion.responseXML.firstChild.tagName == 'parsererror')	// Respuesta vacía
							respuesta = null;
						else{
							respuesta = this.hacerObjeto(peticion.responseXML.firstChild);
							if (respuesta.paxTipo.substr(0,5) == "Excep")
								iu.notificarExcepcionServidor(respuesta);
						}
				}
	  			if( peticion.manejador != null ){ 
					var trozos = peticion.manejador.split(".");
					if (trozos.length == 1)		//Llamada a una función global
						eval(peticion.manejador+"(respuesta)");
					else{				//Llamada a un método de un objeto.
						//Hacemos este berenjenal porque sino, la llamada al método pierde el this.
						var metodo = trozos[trozos.length-1];
						var objeto = trozos[0];
						for (var i=1; i<trozos.length-1; i++)
							objeto += "."+trozos[i];
						//alert(objeto+"."+metodo+".call("+objeto+",respuesta)");
						eval(objeto+"."+metodo+".call("+objeto+",respuesta)");
					}
				}
		  		else 
					return respuesta;
			}
			else
				throw('Excepción AJAX: El estado es '+peticion.statusText);
		}
	};
	this.hacerObjeto = function (nodoXML){
	/**-	Recibe un nodoXML y devuelve un objeto con los atributos del nodoXML en un array indexado.
		Por ahora no admite nodos de texto.
		@param nodoXML El nodo XML a transformar.
	**/
		var objeto = new Object();
		if (!objeto.paxTipo)
			objeto.paxTipo = nodoXML.localName;
		if (nodoXML.hasAttributes())
			//Meto todos los atributos del objeto en un array indexado
			for (var i=0; i<nodoXML.attributes.length; i++)
				objeto[nodoXML.attributes.item(i).nodeName] = nodoXML.attributes.item(i).nodeValue;

		if (nodoXML.hasChildNodes())
			//Cojo cada subnodo y lo proceso
			for (var i=0; i<nodoXML.childNodes.length; i++){
				var nodo = nodoXML.childNodes.item(i);
				if (nodo.nodeType == 3)	//Nodo de Texto
					return nodo.nodeValue;	//TODO: No funcionaría si el nodoXML combina nodos de texto y normales.
				//Miro si ya existe un array con el nombre del elemento
				if (!objeto[nodo.nodeName]) objeto[nodo.nodeName] = new Array();
				objeto[nodo.nodeName].push(this.hacerObjeto(nodo));
			}
		
		return objeto;
	}
};
function Parametro(nombre, valor){
/**- 	Clase que implementa un parámetro.
	Los parámetros se utilizan para enviar datos al servidor. Constan de nombre y valor.
	@param	nombre Texto con el nombre del parámetro.
	@param	valor Texto con el valor del parámetro.
**/
	this.nombre = nombre;
	if (valor == undefined)
		this.valor = '';	//Evitamos los undefined
	else
		this.valor = valor;
}


